package com.gyee.redis2taos.service;

import cn.hutool.core.collection.ListUtil;
import com.gyee.redis2taos.config.RedisConfig;
import com.gyee.redis2taos.config.TaosConfig;
import com.gyee.redis2taos.timeseries.BasicTsPoint;
import com.gyee.redis2taos.timeseries.TsDataType;
import com.gyee.redis2taos.timeseries.TsPoint;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.annotation.Order;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

@Slf4j
@Order(1)
@Service
public class ReadAndWriteService implements ApplicationRunner {

    @Resource
    private RedisService redisService;
    @Resource
    private ThreadPoolTaskExecutor taskExecutor;
    @Resource
    private RedisConfig redisConfig;
    @Resource
    private TaosConfig taosConfig;

    @Override
    public void run(ApplicationArguments args) throws Exception {

        //long l1 = System.currentTimeMillis();
        Set<String> keys = redisService.getKeys();
        //long l2 = System.currentTimeMillis();
        log.info("读取rediskeys成功！！！," + keys.size() + "个点。");
        //按类型分点
        Map<TsDataType, List<BasicTsPoint>> di = keys.stream().map(ss -> {
            if (ss.contains("DI")) {
                return new BasicTsPoint(ss, TsDataType.BOOLEAN);
            } else {
                return new BasicTsPoint(ss, TsDataType.DOUBLE);
            }
        }).collect(Collectors.groupingBy(TsPoint::getTsDataType));
        //di点
        List<String> boolCollect = di.get(TsDataType.BOOLEAN).stream().map(k1 -> k1.getId()).collect(Collectors.toList());
        //ai点
        List<String> douCollect = di.get(TsDataType.DOUBLE).stream().map(k1 -> k1.getId()).collect(Collectors.toList());

        //按数量分组，di点，分为9216个1组，redis单机可以控制在1s内
        List<List<String>> boolStrll = ListUtil.split(boolCollect, 9216);
        //按数量分组，ai点
        List<List<String>> douStrll = ListUtil.split(douCollect, 9216);

        while (true) {
            for (List<String> strings : boolStrll) {
                taskExecutor.execute(new TaskRedis2Taos(strings, redisConfig, taosConfig, TsDataType.BOOLEAN, taskExecutor));
            }
            for (List<String> strings : douStrll) {
                taskExecutor.execute(new TaskRedis2Taos(strings, redisConfig, taosConfig, TsDataType.DOUBLE, taskExecutor));
            }
            Thread.sleep(1000);
            log.info("\n#");
        }
    }
}
