package com.gyee.redis2taos.service;

import com.gyee.redis2taos.config.RedisConfig;
import com.gyee.redis2taos.config.TaosConfig;
import com.gyee.redis2taos.timeseries.TsDataType;
import com.gyee.redis2taos.util.ChangedSave;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.util.StringUtils;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.Pipeline;

import java.sql.Connection;
import java.util.List;
import java.util.Objects;

public class TaskRedis2Taos implements Runnable {
    private final List<String> keys;
    private RedisConfig redisConfig;
    private TaosConfig taosConfig;
    private TsDataType dataType;
    private ThreadPoolTaskExecutor taskExecutor;

    public static final Logger log = LoggerFactory.getLogger(TaskRedis2Taos.class);

    TaskRedis2Taos(List<String> keys, RedisConfig redisConfig, TaosConfig taosConfig, TsDataType dataType, ThreadPoolTaskExecutor taskExecutor) {
        this.keys = keys;
        this.redisConfig = redisConfig;
        this.taosConfig = taosConfig;
        this.dataType = dataType;
        this.taskExecutor = taskExecutor;
    }

    @Override
    public void run() {

        List<Object> objects;

        String pointid = null;

        try (Jedis jedis = redisConfig.getJedis(); Pipeline pipelined = jedis.pipelined();) {
            //TSDBPreparedStatement pstmt = conn.prepareStatement(sql).unwrap(TSDBPreparedStatement.class);

            Connection conn = taosConfig.getInstance();

            for (String key : keys) {
                //pipelined.hscan(key, "0");
                pipelined.hmget(key, "pointid", "value", "timestamp");
            }
            //redis批量获取接收为Object
            objects = pipelined.syncAndReturnAll();

            conn.createStatement().executeUpdate("use " + taosConfig.getDbName() + ";");

            StringBuilder sb = new StringBuilder();
            sb.append("INSERT INTO ");

            String value;
            String datetime;
            double val;
            long ts;
            boolean bl;
            List<String> response;

            if (dataType == TsDataType.BOOLEAN) {
                for (int i = 0; i < objects.size(); ) {
                    response = (List<String>) objects.get(i);

                    value = response.get(1);
                    if (value == null || "".equals(value) || "-nan".equals(value)) continue;
                    datetime = response.get(2);
                    if (StringUtils.isEmpty(datetime)) continue;
                    pointid = response.get(0);
                    if (StringUtils.isEmpty(pointid)) continue;

                    if(ChangedSave.map.containsKey(pointid)){
                        if(!Objects.equals(ChangedSave.map.get(pointid),value)){
                            ChangedSave.map.put(pointid, value);
                            ts = Long.valueOf(datetime) * 1000;
                            bl = "0".equals(value) ? false : true;

                            sb.append(pointid).append(" VALUES (").append(ts).append(",").append(bl).append(") ");
                        }
                    }else {
                        ChangedSave.map.put(pointid, value);
                    }

                    i++;
                    //分为1000个1组，taos单机可以控制在1s内
                    if (i % 1000 == 0 || i == objects.size()) {
                        sb.append(";");

                        taskExecutor.execute(new TaskInsert2Taos(conn, sb.toString(), "USING " +
                                taosConfig.getDIStableName() + " TAGS('','','','','',1,'') VALUES"));

                        sb = new StringBuilder();
                        sb.append("INSERT INTO ");
                    }
                }
            } else {
                for (int i = 0; i < objects.size(); ) {
                    response = (List<String>) objects.get(i);

                    value = response.get(1);
                    if (value == null || "".equals(value) || "-nan".equals(value)) continue;
                    datetime = response.get(2);
                    if (StringUtils.isEmpty(datetime)) continue;
                    pointid = response.get(0);
                    if (StringUtils.isEmpty(pointid)) continue;

                    if(ChangedSave.map.containsKey(pointid)){
                        if(!Objects.equals(ChangedSave.map.get(pointid),value)){
                            ChangedSave.map.put(pointid, value);
                            ts = Long.valueOf(datetime) * 1000;
                            val = Double.valueOf(value);

                            sb.append(pointid).append(" VALUES (").append(ts).append(",").append(val).append(") ");
                        }
                    }else {
                        ChangedSave.map.put(pointid, value);
                    }

                    i++;
                    if (i % 1000 == 0 || i == objects.size()) {
                        sb.append(";");

                        taskExecutor.execute(new TaskInsert2Taos(conn, sb.toString(), "USING " +
                                taosConfig.getAIStableName() + " TAGS('','','','','',1,'') VALUES"));

                        sb = new StringBuilder();
                        sb.append("INSERT INTO ");
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error(pointid + "------读取redis出错！");
        }
        System.out.print("*");
    }
}
